@@ -29,7 +29,8 @@
 extern int filter_expr_debug;
 int filter_expr_parse(CfgLexer *lexer, FilterExprNode **node, gpointer arg);
 
-static CfgLexerKeyword filter_expr_keywords[] = {
+static CfgLexerKeyword filter_expr_keywords[] =
+{
   { "or",                 KW_OR },
   { "and",                KW_AND },
   { "not",                KW_NOT },
@@ -46,18 +47,19 @@ static CfgLexerKeyword filter_expr_keywords[] = {
   { "!=",                 KW_NUM_NE },
   { ">=",                 KW_NUM_GE },
   { ">",                  KW_NUM_GT },
-  { "level",              KW_LEVEL },
-  { "priority",           KW_LEVEL },
+  { "severity",           KW_SEVERITY },
+  { "level",              KW_SEVERITY },
+  { "priority",           KW_SEVERITY },
   { "facility",           KW_FACILITY },
-  { "program",		  KW_PROGRAM },
+  { "program",            KW_PROGRAM },
   { "host",               KW_HOST },
   { "message",            KW_MESSAGE },
-  { "match",		  KW_MATCH },
-  { "netmask",		  KW_NETMASK },
-  { "tags",		  KW_TAGS },
+  { "match",              KW_MATCH },
+  { "netmask",            KW_NETMASK },
+  { "tags",               KW_TAGS },
   { "in_list",            KW_IN_LIST },
 #if SYSLOG_NG_ENABLE_IPV6
-  { "netmask6",     KW_NETMASK6 },
+  { "netmask6",           KW_NETMASK6 },
 #endif
 
   { "value",              KW_VALUE },
@@ -77,4 +79,4 @@ CfgParser filter_expr_parser =
   .parse = (gint (*)(CfgLexer *, gpointer *, gpointer)) filter_expr_parse,
 };
 
-CFG_PARSER_IMPLEMENT_LEXER_BINDING(filter_expr_, FilterExprNode **)
+CFG_PARSER_IMPLEMENT_LEXER_BINDING(filter_expr_, FILTER_EXPR_, FilterExprNode **)
