@@ -69,6 +69,14 @@ log_threaded_dest_driver_set_batch_timeout(LogDriver *s, gint batch_timeout)
   self->batch_timeout = batch_timeout;
 }
 
+void
+log_threaded_dest_driver_set_time_reopen(LogDriver *s, time_t time_reopen)
+{
+  LogThreadedDestDriver *self = (LogThreadedDestDriver *) s;
+
+  self->time_reopen = time_reopen;
+}
+
 /* this should be used in combination with LTR_EXPLICIT_ACK_MGMT to actually confirm message delivery. */
 void
 log_threaded_dest_worker_ack_messages(LogThreadedDestWorker *self, gint batch_size)
@@ -771,6 +779,8 @@ _acquire_worker_queue(LogThreadedDestWorker *self)
   if (!self->queue)
     return FALSE;
 
+  log_queue_set_use_backlog(self->queue, TRUE);
+
   return TRUE;
 }
 
@@ -780,10 +790,6 @@ log_threaded_dest_worker_init_method(LogThreadedDestWorker *self)
   if (self->time_reopen == -1)
     self->time_reopen = self->owner->time_reopen;
 
-  if (!_acquire_worker_queue(self))
-    return FALSE;
-
-  log_queue_set_use_backlog(self->queue, TRUE);
   _register_worker_stats(self);
 
   return TRUE;
@@ -930,24 +936,18 @@ _request_worker_exit(gpointer s)
 }
 
 static gboolean
-_start_worker_thread(LogThreadedDestDriver *self)
+_start_worker_thread(LogThreadedDestWorker *self)
 {
-  gint worker_index = self->workers_started;
-  LogThreadedDestWorker *dw = _construct_worker(self, worker_index);
-
   msg_debug("Starting dedicated worker thread",
-            evt_tag_int("worker_index", worker_index),
-            evt_tag_str("driver", self->super.super.id),
-            log_expr_node_location_tag(self->super.super.super.expr_node));
-  g_assert(self->workers[worker_index] == NULL);
-  self->workers[worker_index] = dw;
-  self->workers_started++;
+            evt_tag_int("worker_index", self->worker_index),
+            evt_tag_str("driver", self->owner->super.super.id),
+            log_expr_node_location_tag(self->owner->super.super.super.expr_node));
 
   main_loop_create_worker_thread(_worker_thread,
                                  _request_worker_exit,
-                                 dw, &self->worker_options);
-  _wait_for_startup_finished(dw);
-  return !dw->startup_failure;
+                                 self, &self->owner->worker_options);
+  _wait_for_startup_finished(self);
+  return !self->startup_failure;
 }
 
 void
@@ -1039,6 +1039,25 @@ _format_seqnum_persist_name(LogThreadedDestDriver *self)
   return persist_name;
 }
 
+static gboolean
+_create_workers(LogThreadedDestDriver *self)
+{
+  /* free previous workers array if set to cope with num_workers change */
+  g_free(self->workers);
+  self->workers = g_new0(LogThreadedDestWorker *, self->num_workers);
+
+  for (self->created_workers = 0; self->created_workers < self->num_workers; self->created_workers++)
+    {
+      LogThreadedDestWorker *dw = _construct_worker(self, self->created_workers);
+
+      self->workers[self->created_workers] = dw;
+      if (!_acquire_worker_queue(dw))
+        return FALSE;
+    }
+
+  return TRUE;
+}
+
 gboolean
 log_threaded_dest_driver_init_method(LogPipe *s)
 {
@@ -1050,13 +1069,19 @@ log_threaded_dest_driver_init_method(LogPipe *s)
 
   self->under_termination = FALSE;
 
-  if (cfg && self->time_reopen == -1)
+  if (self->time_reopen == -1)
     self->time_reopen = cfg->time_reopen;
 
-  /* free previous workers array if set to cope with num_workers change */
-  g_free(self->workers);
-  self->workers = g_new0(LogThreadedDestWorker *, self->num_workers);
-  self->workers_started = 0;
+  self->shared_seq_num = GPOINTER_TO_INT(cfg_persist_config_fetch(cfg,
+                                         _format_seqnum_persist_name(self)));
+  if (!self->shared_seq_num)
+    init_sequence_number(&self->shared_seq_num);
+
+  _register_stats(self);
+
+  if (!_create_workers(self))
+    return FALSE;
+
   return TRUE;
 }
 
@@ -1068,26 +1093,13 @@ gboolean
 log_threaded_dest_driver_start_workers(LogPipe *s)
 {
   LogThreadedDestDriver *self = (LogThreadedDestDriver *) s;
-  GlobalConfig *cfg = log_pipe_get_config((LogPipe *) self);
-  gboolean startup_success = TRUE;
 
-  self->shared_seq_num = GPOINTER_TO_INT(cfg_persist_config_fetch(cfg,
-                                         _format_seqnum_persist_name(self)));
-  if (!self->shared_seq_num)
-    init_sequence_number(&self->shared_seq_num);
-
-  _register_stats(self);
-  for (gint i = 0; startup_success && i < self->num_workers; i++)
+  for (gint worker_index = 0; worker_index < self->num_workers; worker_index++)
     {
-      startup_success &= _start_worker_thread(self);
+      if (!_start_worker_thread(self->workers[worker_index]))
+        return FALSE;
     }
-  return startup_success;
-}
-
-static gboolean
-log_threaded_dest_driver_init(LogPipe *s)
-{
-  return log_threaded_dest_driver_init_method(s);
+  return TRUE;
 }
 
 gboolean
@@ -1106,7 +1118,7 @@ log_threaded_dest_driver_deinit_method(LogPipe *s)
 
   if (!_is_worker_compat_mode(self))
     {
-      for (int i = 0; i < self->workers_started; i++)
+      for (int i = 0; i < self->created_workers; i++)
         log_threaded_dest_worker_free(self->workers[i]);
     }
 
@@ -1132,7 +1144,7 @@ log_threaded_dest_driver_init_instance(LogThreadedDestDriver *self, GlobalConfig
 
   self->worker_options.is_output_thread = TRUE;
 
-  self->super.super.super.init = log_threaded_dest_driver_init;
+  self->super.super.super.init = log_threaded_dest_driver_init_method;
   self->super.super.super.deinit = log_threaded_dest_driver_deinit_method;
   self->super.super.super.queue = log_threaded_dest_driver_queue;
   self->super.super.super.free_fn = log_threaded_dest_driver_free;
