@@ -37,7 +37,7 @@
 
 #define PERSIST_FILE_INITIAL_SIZE 16384
 #define PERSIST_STATE_KEY_BLOCK_SIZE 4096
-#define PERSIST_FILE_WATERMARK 4096
+#define PERSIST_FILE_MAX_ENTRY_SIZE 8448
 
 /*
  * The syslog-ng persistent state is a set of name-value pairs,
@@ -209,7 +209,7 @@ _commit_store(PersistState *self)
 static gboolean
 _check_watermark(PersistState *self)
 {
-  return (self->current_ofs + PERSIST_FILE_WATERMARK < self->current_size);
+  return (self->current_ofs + PERSIST_FILE_MAX_ENTRY_SIZE < self->current_size);
 }
 
 static inline gboolean
@@ -227,6 +227,12 @@ persist_state_run_error_handler(PersistState *self)
 
 /* "value" layer that handles memory block allocation in the file, without working with keys */
 
+static inline void
+_check_max_entry_size(guint32 size)
+{
+  g_assert(size + sizeof(PersistValueHeader) <= PERSIST_FILE_MAX_ENTRY_SIZE);
+}
+
 static PersistEntryHandle
 _alloc_value(PersistState *self, guint32 orig_size, gboolean in_use, guint8 version)
 {
@@ -238,6 +244,8 @@ _alloc_value(PersistState *self, guint32 orig_size, gboolean in_use, guint8 vers
   if ((size & 0x7))
     size = ((size >> 3) + 1) << 3;
 
+  _check_max_entry_size(size);
+
   if (!_check_free_space(self, size))
     {
       msg_error("No more free space exhausted in persist file");
@@ -304,8 +312,8 @@ _free_value(PersistState *self, PersistEntryHandle handle)
 
 /* key management */
 
-gboolean
-persist_state_lookup_key(PersistState *self, const gchar *key, PersistEntryHandle *handle)
+static gboolean
+_persist_state_lookup_key(PersistState *self, const gchar *key, PersistEntryHandle *handle)
 {
   PersistEntry *entry;
 
@@ -336,6 +344,42 @@ persist_state_rename_entry(PersistState *self, const gchar *old_key, const gchar
   return FALSE;
 }
 
+
+static void
+_persist_state_copy_entry(PersistState *self, const PersistEntryHandle from, PersistEntryHandle to, gsize size)
+{
+  gpointer entry_from = persist_state_map_entry(self, from);
+  gpointer entry_to = persist_state_map_entry(self, to);
+
+  memcpy(entry_to, entry_from, size);
+
+  persist_state_unmap_entry(self, from);
+  persist_state_unmap_entry(self, to);
+}
+
+gboolean
+persist_state_move_entry(PersistState *self, const gchar *old_key, const gchar *new_key)
+{
+  gsize size;
+  guint8 version;
+  PersistEntryHandle old_handle = persist_state_lookup_entry(self, old_key, &size, &version);
+  if (!old_handle)
+    return FALSE;
+
+  PersistEntryHandle new_handle = persist_state_alloc_entry(self, new_key, size);
+  if (!new_handle)
+    return FALSE;
+
+  _persist_state_copy_entry(self, old_handle, new_handle, size);
+  _free_value(self, old_handle);
+
+  msg_debug("Persistent entry moved",
+            evt_tag_str("from", old_key),
+            evt_tag_str("to", new_key));
+
+  return TRUE;
+}
+
 /*
  * NOTE: can only be called from the main thread (e.g. log_pipe_init/deinit).
  */
@@ -685,6 +729,7 @@ persist_state_map_entry(PersistState *self, PersistEntryHandle handle)
 {
   /* we count the number of mapped entries in order to know if we're
    * safe to remap the file region */
+  g_assert(handle);
   g_mutex_lock(self->mapped_lock);
   self->mapped_counter++;
   g_mutex_unlock(self->mapped_lock);
@@ -710,7 +755,7 @@ persist_state_unmap_entry(PersistState *self, PersistEntryHandle handle)
 static PersistValueHeader *
 _map_header_of_entry(PersistState *self, const gchar *persist_name, PersistEntryHandle *handle)
 {
-  if (!persist_state_lookup_key(self, persist_name, handle))
+  if (!_persist_state_lookup_key(self, persist_name, handle))
     return NULL;
 
   return _map_header_of_entry_from_handle(self, *handle);
@@ -756,12 +801,19 @@ persist_state_lookup_entry(PersistState *self, const gchar *key, gsize *size, gu
     return 0;
 }
 
+gboolean
+persist_state_entry_exists(PersistState *self, const gchar *persist_name)
+{
+  PersistEntryHandle handle;
+  return _persist_state_lookup_key(self, persist_name, &handle);
+}
+
 gboolean
 persist_state_remove_entry(PersistState *self, const gchar *key)
 {
   PersistEntryHandle handle;
 
-  if (!persist_state_lookup_key(self, key, &handle))
+  if (!_persist_state_lookup_key(self, key, &handle))
     return FALSE;
 
   _free_value(self, handle);
@@ -784,11 +836,12 @@ _foreach_entry_func(gpointer key, gpointer value, gpointer userdata)
 
   PersistValueHeader *header = persist_state_map_entry(data->storage, entry->ofs - sizeof(PersistValueHeader));
   gint size = GUINT32_FROM_BE(header->size);
-  persist_state_unmap_entry(data->storage, entry->ofs);
+  persist_state_unmap_entry(data->storage, entry->ofs - sizeof(PersistValueHeader));
 
-  gpointer *state = persist_state_map_entry(data->storage, entry->ofs);
+  PersistEntryHandle original_handler = entry->ofs;
+  gpointer state = persist_state_map_entry(data->storage, entry->ofs);
   data->func(name, size, state, data->userdata);
-  persist_state_unmap_entry(data->storage, entry->ofs);
+  persist_state_unmap_entry(data->storage, original_handler);
 }
 
 void
