@@ -67,6 +67,7 @@ rcptid_restore_entry(void)
     {
       msg_error("Internal error restoring log reader state, stored data is too new",
                 evt_tag_int("version", data->header.version));
+      rcptid_unmap_state();
       return FALSE;
     }
 
@@ -87,7 +88,7 @@ rcptid_create_new_entry(void)
   RcptidState *data;
 
   rcptid_service.persist_handle = persist_state_alloc_entry(rcptid_service.persist_state, "next.rcptid",
-                                  sizeof(RcptidState));
+                                                            sizeof(RcptidState));
   if (!rcptid_service.persist_handle)
     {
       msg_error("Error allocating RCPTID structure in persist-state");
@@ -154,7 +155,7 @@ rcptid_generate_id(void)
 }
 
 /*restore RCTPID from persist file, if possible, else
-  create new enrty point with "next.rcptid" name*/
+  create new entry point with "next.rcptid" name*/
 gboolean
 rcptid_init(PersistState *state, gboolean use_rcptid)
 {
@@ -168,10 +169,16 @@ rcptid_init(PersistState *state, gboolean use_rcptid)
   rcptid_service.persist_state = state;
   rcptid_service.persist_handle = persist_state_lookup_entry(state, "next.rcptid", &size, &version);
 
-  if (rcptid_service.persist_handle)
-    return rcptid_restore_entry();
+  if (rcptid_service.persist_handle && size == sizeof(RcptidState))
+    {
+      return rcptid_restore_entry();
+    }
   else
-    return rcptid_create_new_entry();
+    {
+      if (rcptid_service.persist_handle)
+        msg_warning("rcpt-id: persist state: invalid size, allocating a new one");
+      return rcptid_create_new_entry();
+    }
 }
 
 void
