@@ -1,6 +1,7 @@
 /*
- * Copyright (c) 2013-2014 Balabit
+ * Copyright (c) 2013-2018 Balabit
  * Copyright (c) 2013 Gergely Nagy <algernon@balabit.hu>
+ * Copyright (c) 2018 Kokan <kokaipeter@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,7 +24,7 @@
  */
 #include <stdlib.h>
 
-#include "testutils.h"
+#include <criterion/criterion.h>
 #include "parse-number.h"
 
 static void
@@ -34,8 +35,8 @@ assert_parse_with_suffix(const gchar *str, gint64 expected)
 
   res = parse_number_with_suffix(str, &n);
 
-  assert_gboolean(res, TRUE, "Parsing (w/ suffix) %s failed", str);
-  assert_gint64(n, expected, "Parsing (w/ suffix) %s failed", str);
+  cr_assert_eq(res, TRUE, "Parsing (w/ suffix) %s failed", str);
+  cr_assert_eq(n, expected, "Parsing (w/ suffix) %s failed", str);
 }
 
 static void
@@ -45,7 +46,7 @@ assert_parse_with_suffix_fails(const gchar *str)
   gboolean res;
 
   res = parse_number_with_suffix(str, &n);
-  assert_gboolean(res, FALSE, "Parsing (w/ suffix) %s succeeded, while expecting failure", str);
+  cr_assert_eq(res, FALSE, "Parsing (w/ suffix) %s succeeded, while expecting failure", str);
 }
 
 static void
@@ -56,8 +57,8 @@ assert_parse(const gchar *str, gint64 expected)
 
   res = parse_number(str, &n);
 
-  assert_gboolean(res, TRUE, "Parsing (w/o suffix) %s failed", str);
-  assert_gint64(n, expected, "Parsing (w/o suffix) %s failed", str);
+  cr_assert_eq(res, TRUE, "Parsing (w/o suffix) %s failed", str);
+  cr_assert_eq(n, expected, "Parsing (w/o suffix) %s failed", str);
 }
 
 static void
@@ -68,21 +69,101 @@ assert_parse_fails(const gchar *str)
 
   res = parse_number(str, &n);
 
-  assert_gboolean(res, FALSE, "Parsing (w/o suffix) %s succeeded, while expecting failure", str);
+  cr_assert_eq(res, FALSE, "Parsing (w/o suffix) %s succeeded, while expecting failure", str);
 }
 
 static void
-test_simple_numbers_are_parsed_properly(void)
+assert_parse_dec(const gchar *str, gint64 expected)
+{
+  gint64 n;
+  gboolean res;
+
+  res = parse_dec_number(str, &n);
+
+  cr_assert_eq(res, TRUE, "Parsing (w/o suffix) %s failed", str);
+  cr_assert_eq(n, expected, "Parsing (w/o suffix) %s failed", str);
+}
+
+static void
+assert_parse_dec_fails(const gchar *str)
+{
+  gint64 n;
+  gboolean res;
+
+  res = parse_dec_number(str, &n);
+
+  cr_assert_eq(res, FALSE, "Parsing (w/o suffix) %s succeeded, while expecting failure", str);
+}
+
+Test(parse_number, test_simple_numbers_are_parsed_properly)
+{
+  assert_parse("1234", 1234);
+  assert_parse("+1234", 1234);
+  assert_parse("-1234", -1234);
+}
+
+Test(parse_number, test_c_like_prefixes_select_base)
+{
+  assert_parse("0x20", 32);
+  assert_parse("0xFF", 255);
+  assert_parse("-0x09", -9);
+
+  assert_parse("020", 16);
+  assert_parse("-010", -8);
+  assert_parse_fails("08");
+  assert_parse_fails("0A");
+
+  assert_parse("20", 20);
+  assert_parse_fails("FF");
+  assert_parse_fails("1FF");
+}
+
+Test(parse_number_dec, test_simple_numbers_are_parsed_properly)
+{
+  assert_parse_dec("1234", 1234);
+  assert_parse_dec("+1234", 1234);
+  assert_parse_dec("-1234", -1234);
+}
+
+Test(parse_number_dec, test_c_like_prefixes_select_base)
+{
+  assert_parse_dec_fails("1F20");
+  assert_parse_dec_fails("0x20");
+  assert_parse_dec_fails("0xFF");
+  assert_parse_dec_fails("-0x09");
+
+  assert_parse_dec("020", 20);
+  assert_parse_dec("-010", -10);
+  assert_parse_dec("08", 8);
+  assert_parse_fails("0A");
+
+  assert_parse_dec("20", 20);
+  assert_parse_dec_fails("FF");
+}
+
+Test(parse_number_with_suffix, test_simple_numbers_are_parsed_properly)
 {
   assert_parse_with_suffix("1234", 1234);
   assert_parse_with_suffix("+1234", 1234);
   assert_parse_with_suffix("-1234", -1234);
-
-  assert_parse("1234", 1234);
 }
 
-static void
-test_exponent_suffix_is_parsed_properly(void)
+Test(parse_number_with_suffix, test_c_like_prefixes_select_base)
+{
+  assert_parse_with_suffix("0x20", 32);
+  assert_parse_with_suffix("0xFF", 255);
+  assert_parse_with_suffix("-0x09", -9);
+
+  assert_parse_with_suffix("020", 16);
+  assert_parse_with_suffix("-010", -8);
+  assert_parse_with_suffix_fails("08");
+  assert_parse_with_suffix_fails("0A");
+
+  assert_parse_with_suffix("20", 20);
+  assert_parse_with_suffix_fails("FF");
+}
+
+Test(parse_number_with_suffix, test_exponent_suffix_is_parsed_properly)
 {
   assert_parse_with_suffix("1K", 1000);
   assert_parse_with_suffix("1k", 1000);
@@ -92,8 +173,7 @@ test_exponent_suffix_is_parsed_properly(void)
   assert_parse_with_suffix("1g", 1000 * 1000 * 1000);
 }
 
-static void
-test_byte_units_are_accepted(void)
+Test(parse_number_with_suffix, test_byte_units_are_accepted)
 {
   assert_parse_with_suffix("1b", 1);
   assert_parse_with_suffix("1B", 1);
@@ -105,8 +185,7 @@ test_byte_units_are_accepted(void)
   assert_parse_with_suffix("1gB", 1000 * 1000 * 1000);
 }
 
-static void
-test_base2_is_selected_by_an_i_modifier(void)
+Test(parse_number_with_suffix, test_base2_is_selected_by_an_i_modifier)
 {
   assert_parse_with_suffix("1Kib", 1024);
   assert_parse_with_suffix("1kiB", 1024);
@@ -119,8 +198,7 @@ test_base2_is_selected_by_an_i_modifier(void)
   assert_parse_with_suffix("1024giB", 1024LL * 1024LL * 1024LL * 1024LL);
 }
 
-static void
-test_invalid_formats_are_not_accepted(void)
+Test(parse_number_with_suffix, test_invalid_formats_are_not_accepted)
 {
   assert_parse_with_suffix_fails("1234Z");
   assert_parse_with_suffix_fails("1234kZ");
@@ -128,15 +206,3 @@ test_invalid_formats_are_not_accepted(void)
   assert_parse_with_suffix_fails("1234kiZ");
   assert_parse_fails("1234kiZ");
 }
-
-int
-main(int argc, char *argv[])
-{
-  test_simple_numbers_are_parsed_properly();
-  test_exponent_suffix_is_parsed_properly();
-  test_byte_units_are_accepted();
-  test_base2_is_selected_by_an_i_modifier();
-  test_invalid_formats_are_not_accepted();
-
-  return 0;
-}
