@@ -1,8 +1,8 @@
-/* A Bison parser, made by GNU Bison 3.7.3.  */
+/* A Bison parser, made by GNU Bison 3.7.6.  */
 
 /* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2020 Free Software Foundation,
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2021 Free Software Foundation,
    Inc.
 
    This program is free software: you can redistribute it and/or modify
@@ -16,7 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -89,6 +89,8 @@ extern int afmongodb_debug;
     LL_CONTEXT_CLIENT_PROTO = 17,  /* LL_CONTEXT_CLIENT_PROTO  */
     LL_CONTEXT_SERVER_PROTO = 18,  /* LL_CONTEXT_SERVER_PROTO  */
     LL_CONTEXT_OPTIONS = 19,       /* LL_CONTEXT_OPTIONS  */
+    LL_CONTEXT_CONFIG = 20,        /* LL_CONTEXT_CONFIG  */
+    LL_CONTEXT_MAX = 21,           /* LL_CONTEXT_MAX  */
     KW_SOURCE = 10000,             /* KW_SOURCE  */
     KW_FILTER = 10001,             /* KW_FILTER  */
     KW_PARSER = 10002,             /* KW_PARSER  */
@@ -223,11 +225,13 @@ extern int afmongodb_debug;
     KW_FETCH_NO_DATA_DELAY = 10513, /* KW_FETCH_NO_DATA_DELAY  */
     KW_MONGODB = 10515,            /* KW_MONGODB  */
     KW_URI = 10516,                /* KW_URI  */
-    KW_COLLECTION = 10517          /* KW_COLLECTION  */
+    KW_COLLECTION = 10517,         /* KW_COLLECTION  */
+    KW_WORKERS = 10518             /* KW_WORKERS  */
   };
   typedef enum afmongodb_tokentype afmongodb_token_kind_t;
 #endif
 /* Token kinds.  */
+#define AFMONGODB_EMPTY -2
 #define AFMONGODB_EOF 0
 #define AFMONGODB_error 256
 #define AFMONGODB_UNDEF 10514
@@ -250,6 +254,8 @@ extern int afmongodb_debug;
 #define LL_CONTEXT_CLIENT_PROTO 17
 #define LL_CONTEXT_SERVER_PROTO 18
 #define LL_CONTEXT_OPTIONS 19
+#define LL_CONTEXT_CONFIG 20
+#define LL_CONTEXT_MAX 21
 #define KW_SOURCE 10000
 #define KW_FILTER 10001
 #define KW_PARSER 10002
@@ -385,6 +391,7 @@ extern int afmongodb_debug;
 #define KW_MONGODB 10515
 #define KW_URI 10516
 #define KW_COLLECTION 10517
+#define KW_WORKERS 10518
 
 /* Value type.  */
 #if ! defined AFMONGODB_STYPE && ! defined AFMONGODB_STYPE_IS_DECLARED
