@@ -1,8 +1,8 @@
-/* A Bison parser, made by GNU Bison 3.7.3.  */
+/* A Bison parser, made by GNU Bison 3.7.6.  */
 
 /* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2020 Free Software Foundation,
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2021 Free Software Foundation,
    Inc.
 
    This program is free software: you can redistribute it and/or modify
@@ -16,7 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -81,6 +81,8 @@ extern int csvparser_debug;
     LL_CONTEXT_CLIENT_PROTO = 17,  /* LL_CONTEXT_CLIENT_PROTO  */
     LL_CONTEXT_SERVER_PROTO = 18,  /* LL_CONTEXT_SERVER_PROTO  */
     LL_CONTEXT_OPTIONS = 19,       /* LL_CONTEXT_OPTIONS  */
+    LL_CONTEXT_CONFIG = 20,        /* LL_CONTEXT_CONFIG  */
+    LL_CONTEXT_MAX = 21,           /* LL_CONTEXT_MAX  */
     KW_SOURCE = 10000,             /* KW_SOURCE  */
     KW_FILTER = 10001,             /* KW_FILTER  */
     KW_PARSER = 10002,             /* KW_PARSER  */
@@ -222,11 +224,13 @@ extern int csvparser_debug;
     KW_QUOTE_PAIRS = 10521,        /* KW_QUOTE_PAIRS  */
     KW_NULL = 10522,               /* KW_NULL  */
     KW_CHARS = 10523,              /* KW_CHARS  */
-    KW_STRINGS = 10524             /* KW_STRINGS  */
+    KW_STRINGS = 10524,            /* KW_STRINGS  */
+    KW_DROP_INVALID = 10525        /* KW_DROP_INVALID  */
   };
   typedef enum csvparser_tokentype csvparser_token_kind_t;
 #endif
 /* Token kinds.  */
+#define CSVPARSER_EMPTY -2
 #define CSVPARSER_EOF 0
 #define CSVPARSER_error 256
 #define CSVPARSER_UNDEF 10514
@@ -249,6 +253,8 @@ extern int csvparser_debug;
 #define LL_CONTEXT_CLIENT_PROTO 17
 #define LL_CONTEXT_SERVER_PROTO 18
 #define LL_CONTEXT_OPTIONS 19
+#define LL_CONTEXT_CONFIG 20
+#define LL_CONTEXT_MAX 21
 #define KW_SOURCE 10000
 #define KW_FILTER 10001
 #define KW_PARSER 10002
@@ -391,6 +397,7 @@ extern int csvparser_debug;
 #define KW_NULL 10522
 #define KW_CHARS 10523
 #define KW_STRINGS 10524
+#define KW_DROP_INVALID 10525
 
 /* Value type.  */
 #if ! defined CSVPARSER_STYPE && ! defined CSVPARSER_STYPE_IS_DECLARED
