@@ -47,6 +47,8 @@
 
 #define PATH_QDISK              PATH_LOCALSTATEDIR
 
+#define QDISK_HDR_VERSION_CURRENT 2
+
 typedef union _QDiskFileHeader
 {
   struct
@@ -65,6 +67,8 @@ typedef union _QDiskFileHeader
     QDiskQueuePosition qoverflow_pos;
     gint64 backlog_head;
     gint64 backlog_len;
+
+    guint8 use_v1_wrap_condition;
   };
   gchar _pad2[QDISK_RESERVED_SPACE];
 } QDiskFileHeader;
@@ -100,15 +104,26 @@ pwrite_strict(gint fd, const void *buf, size_t count, off_t offset)
 
 
 static gboolean
-_is_position_eof(QDisk *self, gint64 position)
+_is_position_after_disk_buf_size(QDisk *self, gint64 position)
 {
-  return position >= self->file_size;
+  return position > self->options->disk_buf_size;
 }
 
 static guint64
-_correct_position_if_eof(QDisk *self, gint64 *position)
+_correct_position_if_after_disk_buf_size(QDisk *self, gint64 *position)
 {
-  if (_is_position_eof(self, *position))
+  if (G_UNLIKELY(self->hdr->use_v1_wrap_condition))
+    {
+      gboolean position_is_eof = *position >= self->file_size;
+      if (position_is_eof)
+        {
+          *position = QDISK_RESERVED_SPACE;
+          self->hdr->use_v1_wrap_condition = FALSE;
+        }
+      return *position;
+    }
+
+  if (_is_position_after_disk_buf_size(self, *position))
     {
       *position = QDISK_RESERVED_SPACE;
     }
@@ -154,10 +169,9 @@ qdisk_started(QDisk *self)
 static inline gboolean
 _is_qdisk_overwritten(QDisk *self)
 {
-  return self->hdr->write_head > self->options->disk_buf_size;
+  return _is_position_after_disk_buf_size(self, self->hdr->write_head);
 }
 
-
 static inline gboolean
 _is_backlog_head_prevent_write_head(QDisk *self)
 {
@@ -193,6 +207,14 @@ qdisk_is_space_avail(QDisk *self, gint at_least)
 {
   /* sizeof(guint32): record_length is a 4 bytes long value which is stored before each serialized LogMessage */
   gint64 msg_len = at_least + sizeof(guint32);
+  /* write follows read (e.g. we are appending to the file) OR
+   * there's enough space between write and read.
+   *
+   * If write follows read we need to check two things:
+   *   - either we are below the maximum limit (GINT64_FROM_BE(self->hdr->write_head) < self->options->disk_buf_size)
+   *   - or we can wrap around (GINT64_FROM_BE(self->hdr->read_head) != QDISK_RESERVED_SPACE)
+   * If neither of the above is true, the buffer is full.
+   */
   return (
            (_is_backlog_head_prevent_write_head(self)) &&
            (_is_write_head_less_than_max_size(self) || _is_able_to_reset_write_head_to_beginning_of_qdisk(self))
@@ -201,35 +223,54 @@ qdisk_is_space_avail(QDisk *self, gint at_least)
 }
 
 static gboolean
-_truncate_file(QDisk *self, off_t new_size)
+_ftruncate_would_reduce_file(QDisk *self, gint64 expected_size)
 {
-  gboolean success = TRUE;
+  gint64 expected_size_change = expected_size - self->file_size;
+  return expected_size_change < 0;
+}
 
-  if (ftruncate(self->fd, new_size) < 0)
+static gboolean
+_possible_size_reduction_reaches_truncate_threshold(QDisk *self, gint64 expected_size)
+{
+  gint64 possible_size_reduction = self->file_size - expected_size;
+  gint64 truncate_threshold = (gint64)(qdisk_get_maximum_size(self) * self->options->truncate_size_ratio);
+  return possible_size_reduction >= truncate_threshold;
+}
+
+static void
+_maybe_truncate_file(QDisk *self, gint64 expected_size)
+{
+  if (_ftruncate_would_reduce_file(self, expected_size) &&
+      !_possible_size_reduction_reaches_truncate_threshold(self, expected_size) &&
+      G_LIKELY(!self->hdr->use_v1_wrap_condition))
     {
-      success = FALSE;
-      off_t file_size = -1;
-
-      struct stat st;
-      if (fstat(self->fd, &st) < 0)
-        {
-          msg_error("truncate file: cannot stat",
-                    evt_tag_error("error"));
-        }
-      else
-        {
-          file_size = st.st_size;
-        }
-
-      msg_error("Error truncating disk-queue file",
-                evt_tag_error("error"),
-                evt_tag_str("filename", self->filename),
-                evt_tag_long("expected-size", new_size),
-                evt_tag_long("file_size", file_size),
-                evt_tag_int("fd", self->fd));
+      return;
     }
 
-  return success;
+  msg_debug("Truncating queue file", evt_tag_str("filename", self->filename), evt_tag_long("new size", expected_size));
+
+  if (ftruncate(self->fd, (off_t) expected_size) == 0)
+    {
+      self->file_size = expected_size;
+      return;
+    }
+
+  struct stat st;
+  if (fstat(self->fd, &st) < 0)
+    {
+      msg_error("truncate file: cannot stat", evt_tag_error("error"));
+    }
+  else
+    {
+      self->file_size = (gint64) st.st_size;
+    }
+
+  msg_error("Error truncating disk-queue file",
+            evt_tag_error("error"),
+            evt_tag_str("filename", self->filename),
+            evt_tag_long("expected-size", expected_size),
+            evt_tag_long("file-size", self->file_size),
+            evt_tag_int("fd", self->fd));
 }
 
 static gint64
@@ -250,22 +291,19 @@ qdisk_get_lowest_used_queue_offset(QDisk *self)
 }
 
 static void
-qdisk_try_to_truncate_file_to_minimal(QDisk *self, gint64 *new_file_end_offset)
+_truncate_file_to_minimal(QDisk *self)
 {
-  gint64 file_end_offset = 0;
   if (qdisk_is_file_empty(self))
     {
-      _truncate_file(self, QDISK_RESERVED_SPACE);
-      file_end_offset = QDISK_RESERVED_SPACE;
+      _maybe_truncate_file(self, QDISK_RESERVED_SPACE);
+      return;
     }
-  else
-    {
-      file_end_offset = qdisk_get_lowest_used_queue_offset(self);
-      if(file_end_offset > QDISK_RESERVED_SPACE)
-        _truncate_file(self, file_end_offset);
-    }
-  if (new_file_end_offset)
-    *new_file_end_offset = file_end_offset;
+
+  gint64 file_end_offset = qdisk_get_lowest_used_queue_offset(self);
+  if (file_end_offset <= QDISK_RESERVED_SPACE)
+    return;
+
+  _maybe_truncate_file(self, file_end_offset);
 }
 
 
@@ -284,18 +322,25 @@ qdisk_get_empty_space(QDisk *self)
   return bpos - wpos;
 }
 
+static gboolean
+_could_not_wrap_write_head_last_push_but_now_can(QDisk *self)
+{
+  return _is_qdisk_overwritten(self) && _is_able_to_reset_write_head_to_beginning_of_qdisk(self);
+}
+
 gboolean
 qdisk_push_tail(QDisk *self, GString *record)
 {
+  if (_could_not_wrap_write_head_last_push_but_now_can(self))
+    {
+      /*
+       * We can safely move the write_head to the beginning, but still
+       * not sure, if this message will have space. We move the write_head
+       * then check the available space compared to the new position.
+       */
+      self->hdr->write_head = QDISK_RESERVED_SPACE;
+    }
 
-  /* write follows read (e.g. we are appending to the file) OR
-   * there's enough space between write and read.
-   *
-   * If write follows read we need to check two things:
-   *   - either we are below the maximum limit (GINT64_FROM_BE(self->hdr->write_head) < self->options->disk_buf_size)
-   *   - or we can wrap around (GINT64_FROM_BE(self->hdr->read_head) != QDISK_RESERVED_SPACE)
-   * If neither of the above is true, the buffer is full.
-   */
   if (!qdisk_is_space_avail(self, record->len))
     return FALSE;
 
@@ -344,20 +389,23 @@ qdisk_push_tail(QDisk *self, GString *record)
     {
       if (self->file_size > self->hdr->write_head)
         {
-          msg_debug("Unused area ahead of write_head, truncate queue file",
-                    evt_tag_long("new size",  self->hdr->write_head));
-          _truncate_file(self, self->hdr->write_head);
+          _maybe_truncate_file(self, self->hdr->write_head);
+        }
+      else
+        {
+          self->file_size = self->hdr->write_head;
         }
-      self->file_size = self->hdr->write_head;
 
-      if (_is_qdisk_overwritten(self) && self->hdr->backlog_head  != QDISK_RESERVED_SPACE)
+      if (_is_qdisk_overwritten(self) && _is_able_to_reset_write_head_to_beginning_of_qdisk(self))
         {
           /* we were appending to the file, we are over the limit, and space
            * is available before the read head. truncate and wrap.
            *
-           * Otherwise we let the write_head over size limits for a bit and
-           * for the next message, the condition at the beginning of this
-           * function will cause the push to fail */
+           * Otherwise try to wrap again in the beginning of the next push.
+           *
+           * This way we guarantee, that only a part of 1 message is written after
+           * disk_buf_size.
+           */
           self->hdr->write_head = QDISK_RESERVED_SPACE;
         }
     }
@@ -429,7 +477,7 @@ qdisk_pop_head(QDisk *self, GString *record)
 
       if (self->hdr->read_head > self->hdr->write_head)
         {
-          self->hdr->read_head = _correct_position_if_eof(self, &self->hdr->read_head);
+          self->hdr->read_head = _correct_position_if_after_disk_buf_size(self, &self->hdr->read_head);
         }
 
       self->hdr->length--;
@@ -438,15 +486,9 @@ qdisk_pop_head(QDisk *self, GString *record)
           self->hdr->backlog_head = self->hdr->read_head;
 
           g_assert(self->hdr->backlog_len == 0);
-          if (!self->options->read_only && qdisk_is_file_empty(self))
+          if (!self->options->read_only)
             {
-              msg_debug("Queue file became empty, truncating file",
-                        evt_tag_str("filename", self->filename));
-              self->hdr->read_head = QDISK_RESERVED_SPACE;
-              self->hdr->write_head = QDISK_RESERVED_SPACE;
-              self->hdr->backlog_head = self->hdr->read_head;
-              self->hdr->length = 0;
-              _truncate_file(self, self->hdr->write_head);
+              qdisk_reset_file_if_empty(self);
             }
         }
       return TRUE;
@@ -567,6 +609,22 @@ _load_non_reliable_queues(QDisk *self, GQueue *qout, GQueue *qbacklog, GQueue *q
 
 #define _clear(obj) memset(&obj, 0, sizeof(obj));
 
+static gint64
+_number_of_messages(QDisk *self)
+{
+  if (self->options->reliable)
+    {
+      return self->hdr->length + self->hdr->backlog_len;
+    }
+  else
+    {
+      return self->hdr->length +
+             self->hdr->qbacklog_pos.count +
+             self->hdr->qout_pos.count +
+             self->hdr->qoverflow_pos.count;
+    }
+}
+
 static void
 _reset_queue_pointers(QDisk *self)
 {
@@ -609,19 +667,24 @@ _load_state(QDisk *self, GQueue *qout, GQueue *qbacklog, GQueue *qoverflow)
       if (!_load_non_reliable_queues(self, qout, qbacklog, qoverflow))
         return FALSE;
 
-      gint64 end_ofs = QDISK_RESERVED_SPACE;
+      self->file_size = QDISK_RESERVED_SPACE;
       if (!self->options->read_only)
         {
-          qdisk_try_to_truncate_file_to_minimal(self, &end_ofs);
+          _truncate_file_to_minimal(self);
         }
-      self->file_size = MAX(end_ofs, QDISK_RESERVED_SPACE);
 
       msg_info("Disk-buffer state loaded",
                evt_tag_str("filename", self->filename),
-               evt_tag_long("qout_length", self->hdr->qout_pos.count),
-               evt_tag_long("qbacklog_length", self->hdr->qbacklog_pos.count),
-               evt_tag_long("qoverflow_length", self->hdr->qoverflow_pos.count),
-               evt_tag_long("qdisk_length", self->hdr->length));
+               evt_tag_long("number_of_messages", _number_of_messages(self)));
+
+      msg_debug("Disk-buffer internal state",
+                evt_tag_str("filename", self->filename),
+                evt_tag_long("qout_length", self->hdr->qout_pos.count),
+                evt_tag_long("qbacklog_length", self->hdr->qbacklog_pos.count),
+                evt_tag_long("qoverflow_length", self->hdr->qoverflow_pos.count),
+                evt_tag_long("qdisk_length", self->hdr->length),
+                evt_tag_long("read_head", self->hdr->read_head),
+                evt_tag_long("write_head", self->hdr->write_head));
 
       _reset_queue_pointers(self);
     }
@@ -632,15 +695,15 @@ _load_state(QDisk *self, GQueue *qout, GQueue *qbacklog, GQueue *qoverflow)
       self->file_size = st.st_size;
       msg_info("Reliable disk-buffer state loaded",
                evt_tag_str("filename", self->filename),
-               evt_tag_long("queue_length", self->hdr->length),
-               evt_tag_long("size", self->hdr->write_head - self->hdr->read_head));
+               evt_tag_long("number_of_messages", _number_of_messages(self)));
 
       msg_debug("Reliable disk-buffer internal state",
                 evt_tag_str("filename", self->filename),
+                evt_tag_long("queue_length", self->hdr->length),
+                evt_tag_long("backlog_len", self->hdr->backlog_len),
                 evt_tag_long("backlog_head", self->hdr->backlog_head),
                 evt_tag_long("read_head", self->hdr->read_head),
-                evt_tag_long("write_head", self->hdr->write_head),
-                evt_tag_long("backlog_len", self->hdr->backlog_len));
+                evt_tag_long("write_head", self->hdr->write_head));
     }
 
   return TRUE;
@@ -768,15 +831,6 @@ qdisk_save_state(QDisk *self, GQueue *qout, GQueue *qbacklog, GQueue *qoverflow)
   return TRUE;
 }
 
-static void
-_update_header_with_default_values(QDisk *self)
-{
-  self->hdr->big_endian = TRUE;
-  self->hdr->version = 1;
-  self->hdr->backlog_head = self->hdr->read_head;
-  self->hdr->backlog_len = 0;
-}
-
 static gboolean
 _create_path(const gchar *filename)
 {
@@ -785,6 +839,32 @@ _create_path(const gchar *filename)
   return file_perm_options_create_containing_directory(&fpermoptions, filename);
 }
 
+static gboolean
+_is_header_version_current(QDisk *self)
+{
+  return self->hdr->version == QDISK_HDR_VERSION_CURRENT;
+}
+
+static void
+_upgrade_header(QDisk *self)
+{
+  if (self->hdr->version == 0)
+    {
+      self->hdr->big_endian = TRUE;
+      self->hdr->backlog_head = self->hdr->read_head;
+      self->hdr->backlog_len = 0;
+    }
+
+  if (self->hdr->version < 2)
+    {
+      struct stat st;
+      gboolean file_was_overwritten = (fstat(self->fd, &st) != 0 || st.st_size > self->options->disk_buf_size);
+      self->hdr->use_v1_wrap_condition = file_was_overwritten;
+    }
+
+  self->hdr->version = QDISK_HDR_VERSION_CURRENT;
+}
+
 gboolean
 qdisk_start(QDisk *self, const gchar *filename, GQueue *qout, GQueue *qbacklog, GQueue *qoverflow)
 {
@@ -883,13 +963,14 @@ qdisk_start(QDisk *self, const gchar *filename, GQueue *qout, GQueue *qbacklog,
           self->fd = -1;
           return FALSE;
         }
-      self->hdr->version = 1;
+      self->hdr->version = QDISK_HDR_VERSION_CURRENT;
       self->hdr->big_endian = (G_BYTE_ORDER == G_BIG_ENDIAN);
 
       self->hdr->read_head = QDISK_RESERVED_SPACE;
       self->hdr->write_head = QDISK_RESERVED_SPACE;
       self->hdr->backlog_head = self->hdr->read_head;
       self->hdr->length = 0;
+      self->hdr->use_v1_wrap_condition = FALSE;
       self->file_size = self->hdr->write_head;
 
       if (!qdisk_save_state(self, qout, qbacklog, qoverflow))
@@ -917,8 +998,11 @@ qdisk_start(QDisk *self, const gchar *filename, GQueue *qout, GQueue *qbacklog,
           self->fd = -1;
           return FALSE;
         }
-      if (self->hdr->version == 0)
-        _update_header_with_default_values(self);
+
+      if (!_is_header_version_current(self))
+        {
+          _upgrade_header(self);
+        }
 
       if ((self->hdr->big_endian && G_BYTE_ORDER == G_LITTLE_ENDIAN) ||
           (!self->hdr->big_endian && G_BYTE_ORDER == G_BIG_ENDIAN))
@@ -1013,21 +1097,22 @@ qdisk_skip_record(QDisk *self, guint64 position)
   new_position += record_length + sizeof(record_length);
   if (new_position > self->hdr->write_head)
     {
-      new_position = _correct_position_if_eof(self, (gint64 *)&new_position);
+      new_position = _correct_position_if_after_disk_buf_size(self, (gint64 *)&new_position);
     }
   return new_position;
 }
 
 void
-qdisk_reset_file_if_possible(QDisk *self)
+qdisk_reset_file_if_empty(QDisk *self)
 {
-  if (qdisk_is_file_empty(self))
-    {
-      self->hdr->read_head = QDISK_RESERVED_SPACE;
-      self->hdr->write_head = QDISK_RESERVED_SPACE;
-      self->hdr->backlog_head = QDISK_RESERVED_SPACE;
-      _truncate_file (self, QDISK_RESERVED_SPACE);
-    }
+  if (!qdisk_is_file_empty(self))
+    return;
+
+  self->hdr->read_head = QDISK_RESERVED_SPACE;
+  self->hdr->write_head = QDISK_RESERVED_SPACE;
+  self->hdr->backlog_head = QDISK_RESERVED_SPACE;
+
+  _maybe_truncate_file(self, QDISK_RESERVED_SPACE);
 }
 
 DiskQueueOptions *
