@@ -1,8 +1,8 @@
-/* A Bison parser, made by GNU Bison 3.7.3.  */
+/* A Bison parser, made by GNU Bison 3.7.6.  */
 
 /* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2020 Free Software Foundation,
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2021 Free Software Foundation,
    Inc.
 
    This program is free software: you can redistribute it and/or modify
@@ -16,7 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -81,6 +81,8 @@ extern int threaded_random_generator_debug;
     LL_CONTEXT_CLIENT_PROTO = 17,  /* LL_CONTEXT_CLIENT_PROTO  */
     LL_CONTEXT_SERVER_PROTO = 18,  /* LL_CONTEXT_SERVER_PROTO  */
     LL_CONTEXT_OPTIONS = 19,       /* LL_CONTEXT_OPTIONS  */
+    LL_CONTEXT_CONFIG = 20,        /* LL_CONTEXT_CONFIG  */
+    LL_CONTEXT_MAX = 21,           /* LL_CONTEXT_MAX  */
     KW_SOURCE = 10000,             /* KW_SOURCE  */
     KW_FILTER = 10001,             /* KW_FILTER  */
     KW_PARSER = 10002,             /* KW_PARSER  */
@@ -220,6 +222,7 @@ extern int threaded_random_generator_debug;
   typedef enum threaded_random_generator_tokentype threaded_random_generator_token_kind_t;
 #endif
 /* Token kinds.  */
+#define THREADED_RANDOM_GENERATOR_EMPTY -2
 #define THREADED_RANDOM_GENERATOR_EOF 0
 #define THREADED_RANDOM_GENERATOR_error 256
 #define THREADED_RANDOM_GENERATOR_UNDEF 10514
@@ -242,6 +245,8 @@ extern int threaded_random_generator_debug;
 #define LL_CONTEXT_CLIENT_PROTO 17
 #define LL_CONTEXT_SERVER_PROTO 18
 #define LL_CONTEXT_OPTIONS 19
+#define LL_CONTEXT_CONFIG 20
+#define LL_CONTEXT_MAX 21
 #define KW_SOURCE 10000
 #define KW_FILTER 10001
 #define KW_PARSER 10002
