@@ -231,10 +231,9 @@ extern int kafka_debug;
     KW_FLUSH_TIMEOUT_ON_SHUTDOWN = 10520, /* KW_FLUSH_TIMEOUT_ON_SHUTDOWN  */
     KW_FLUSH_TIMEOUT_ON_RELOAD = 10521, /* KW_FLUSH_TIMEOUT_ON_RELOAD  */
     KW_POLL_TIMEOUT = 10522,       /* KW_POLL_TIMEOUT  */
-    KW_CLIENT_LIB_DIR = 10523,     /* KW_CLIENT_LIB_DIR  */
-    KW_BOOTSTRAP_SERVERS = 10524,  /* KW_BOOTSTRAP_SERVERS  */
-    KW_SYNC_SEND = 10525,          /* KW_SYNC_SEND  */
-    KW_PROPERTIES_FILE = 10526     /* KW_PROPERTIES_FILE  */
+    KW_BOOTSTRAP_SERVERS = 10523,  /* KW_BOOTSTRAP_SERVERS  */
+    KW_SYNC_SEND = 10524,          /* KW_SYNC_SEND  */
+    KW_PROPERTIES_FILE = 10525     /* KW_PROPERTIES_FILE  */
   };
   typedef enum kafka_tokentype kafka_token_kind_t;
 #endif
@@ -404,10 +403,9 @@ extern int kafka_debug;
 #define KW_FLUSH_TIMEOUT_ON_SHUTDOWN 10520
 #define KW_FLUSH_TIMEOUT_ON_RELOAD 10521
 #define KW_POLL_TIMEOUT 10522
-#define KW_CLIENT_LIB_DIR 10523
-#define KW_BOOTSTRAP_SERVERS 10524
-#define KW_SYNC_SEND 10525
-#define KW_PROPERTIES_FILE 10526
+#define KW_BOOTSTRAP_SERVERS 10523
+#define KW_SYNC_SEND 10524
+#define KW_PROPERTIES_FILE 10525
 
 /* Value type.  */
 #if ! defined KAFKA_STYPE && ! defined KAFKA_STYPE_IS_DECLARED
