@@ -28,9 +28,11 @@
 #include "mainloop-io-worker.h"
 #include "persist-state.h"
 #include "persistable-state-header.h"
-#include "ack_tracker.h"
+#include "ack-tracker/ack_tracker.h"
 #include "parse-number.h"
 #include "journal-reader.h"
+#include "timeutils/misc.h"
+#include "ack-tracker/ack_tracker_factory.h"
 
 #include <stdlib.h>
 #include <iv_event.h>
@@ -243,13 +245,13 @@ _set_message_timestamp(JournalReader *self, LogMessage *msg)
   guint64 ts;
 
   journald_get_realtime_usec(self->journal, &ts);
-  msg->timestamps[LM_TS_STAMP].tv_sec = ts / 1000000;
-  msg->timestamps[LM_TS_STAMP].tv_usec = ts % 1000000;
-  msg->timestamps[LM_TS_STAMP].zone_offset = time_zone_info_get_offset(self->options->recv_time_zone_info,
-                                             msg->timestamps[LM_TS_STAMP].tv_sec);
-  if (msg->timestamps[LM_TS_STAMP].zone_offset == -1)
+  msg->timestamps[LM_TS_STAMP].ut_sec = ts / 1000000;
+  msg->timestamps[LM_TS_STAMP].ut_usec = ts % 1000000;
+  msg->timestamps[LM_TS_STAMP].ut_gmtoff = time_zone_info_get_offset(self->options->recv_time_zone_info,
+                                           msg->timestamps[LM_TS_STAMP].ut_sec);
+  if (msg->timestamps[LM_TS_STAMP].ut_gmtoff == -1)
     {
-      msg->timestamps[LM_TS_STAMP].zone_offset = get_local_timezone_ofs(msg->timestamps[LM_TS_STAMP].tv_sec);
+      msg->timestamps[LM_TS_STAMP].ut_gmtoff = get_local_timezone_ofs(msg->timestamps[LM_TS_STAMP].ut_sec);
     }
 }
 
@@ -516,11 +518,10 @@ _work_finished(gpointer s)
     {
       _update_watches(self);
     }
-  log_pipe_unref(&self->super.super);
 }
 
 static void
-_work_perform(gpointer s)
+_work_perform(gpointer s, GIOCondition cond)
 {
   JournalReader *self = (JournalReader *) s;
   self->notify_code = _fetch_log(self);
@@ -545,16 +546,15 @@ _io_process_input(gpointer s)
   JournalReader *self = (JournalReader *) s;
 
   _stop_watches(self);
-  log_pipe_ref(&self->super.super);
   if ((self->options->flags & JR_THREADED))
     {
-      main_loop_io_worker_job_submit(&self->io_job);
+      main_loop_io_worker_job_submit(&self->io_job, G_IO_IN);
     }
   else
     {
       if (!main_loop_worker_job_quit())
         {
-          _work_perform(s);
+          _work_perform(s, G_IO_IN);
           _work_finished(s);
         }
     }
@@ -585,6 +585,63 @@ _add_poll_events(JournalReader *self)
   return TRUE;
 }
 
+#if SYSLOG_NG_HAVE_JOURNAL_NAMESPACES
+static void
+_journal_process_namespace(gchar *namespace_option, gchar **journal_namespace, gint *journal_flags)
+{
+  if (strcmp(namespace_option, "*") == 0)
+    {
+      *journal_flags |= SD_JOURNAL_ALL_NAMESPACES;
+      if (strlen(namespace_option) > 1)
+        {
+          msg_warning("namespace('*'): discarding everything after the asterisk");
+        }
+    }
+  else if (strncmp(namespace_option, "+", 1) == 0)
+    {
+      *journal_flags |= SD_JOURNAL_INCLUDE_DEFAULT_NAMESPACE;
+      if (strlen(namespace_option) > 1)
+        {
+          *journal_namespace = namespace_option + 1;
+        }
+    }
+  else
+    {
+      if (strlen(namespace_option) > 0)
+        {
+          *journal_namespace = namespace_option;
+        }
+    }
+}
+#endif
+
+static gint
+_journal_open(JournalReader *self)
+{
+  gint journal_flags = SD_JOURNAL_LOCAL_ONLY;
+
+#if SYSLOG_NG_HAVE_JOURNAL_NAMESPACES
+  gchar *journal_namespace = NULL;
+  _journal_process_namespace(self->options->namespace, &journal_namespace, &journal_flags);
+  return journald_open_namespace(self->journal, journal_namespace, journal_flags);
+#else
+  return journald_open(self->journal, journal_flags);
+#endif
+}
+
+static void
+_init_persist_name(JournalReader *self)
+{
+#if SYSLOG_NG_HAVE_JOURNAL_NAMESPACES
+  if (strcmp(self->options->namespace, "*") != 0)
+    {
+      self->persist_name = g_strdup_printf("systemd_journal(%s)", self->options->namespace);
+      return;
+    }
+#endif
+  self->persist_name = g_strdup("systemd-journal");
+}
+
 static gboolean
 _init(LogPipe *s)
 {
@@ -596,10 +653,12 @@ _init(LogPipe *s)
       return FALSE;
     }
 
+  _init_persist_name(self);
+
   if (!log_source_init(s))
     return FALSE;
 
-  gint res = journald_open(self->journal, SD_JOURNAL_LOCAL_ONLY);
+  gint res = _journal_open(self);
   if (res < 0)
     {
       msg_error("Error opening the journal",
@@ -653,7 +712,8 @@ journal_reader_set_options(LogPipe *s, LogPipe *control, JournalReaderOptions *o
   JournalReader *self = (JournalReader *) s;
 
   log_source_set_options(&self->super, &options->super, stats_id, stats_instance,
-                         (options->flags & JR_THREADED), TRUE, control->expr_node);
+                         (options->flags & JR_THREADED), control->expr_node);
+  log_source_set_ack_tracker_factory(&self->super, consecutive_ack_tracker_factory_new());
 
   log_pipe_unref(self->control);
   log_pipe_ref(control);
@@ -675,8 +735,10 @@ _init_watches(JournalReader *self)
 
   main_loop_io_worker_job_init(&self->io_job);
   self->io_job.user_data = self;
-  self->io_job.work = (void (*)(void *)) _work_perform;
+  self->io_job.work = (void (*)(void *, GIOCondition)) _work_perform;
   self->io_job.completion = (void (*)(void *)) _work_finished;
+  self->io_job.engage = (void (*)(void *)) log_pipe_ref;
+  self->io_job.release = (void (*)(void *)) log_pipe_unref;
 }
 
 JournalReader *
@@ -688,7 +750,7 @@ journal_reader_new(GlobalConfig *cfg, Journald *journal)
   self->super.super.init = _init;
   self->super.super.deinit = _deinit;
   self->super.super.free_fn = _free;
-  self->persist_name = g_strdup("systemd-journal");
+  self->persist_name = NULL;
   self->journal = journal;
   _init_watches(self);
   return self;
@@ -724,15 +786,79 @@ journal_reader_options_init(JournalReaderOptions *options, GlobalConfig *cfg, co
           options->prefix = g_strdup(default_prefix);
         }
     }
+#if SYSLOG_NG_HAVE_JOURNAL_NAMESPACES
+  if (options->namespace == NULL)
+    {
+      const gchar *default_namespace = "*";
+      options->namespace = g_strdup(default_namespace);
+    }
+#else
+  if (options->namespace)
+    {
+      msg_warning("namespace() option on systemd-journal() will have no effect! (systemd < v245)");
+    }
+#endif
   options->initialized = TRUE;
 }
 
+void
+journal_reader_options_set_default_severity(JournalReaderOptions *self, gint severity)
+{
+  if (self->default_pri == 0xFFFF)
+    self->default_pri = LOG_USER;
+  self->default_pri = (self->default_pri & ~7) | severity;
+}
+
+void
+journal_reader_options_set_default_facility(JournalReaderOptions *self, gint facility)
+{
+  if (self->default_pri == 0xFFFF)
+    self->default_pri = LOG_NOTICE;
+  self->default_pri = (self->default_pri & 7) | facility;
+}
+
+void
+journal_reader_options_set_time_zone(JournalReaderOptions *self, gchar *time_zone)
+{
+  if (self->recv_time_zone)
+    g_free(self->recv_time_zone);
+  self->recv_time_zone = g_strdup(time_zone);
+}
+
+void
+journal_reader_options_set_prefix(JournalReaderOptions *self, gchar *prefix)
+{
+  if (self->prefix)
+    g_free(self->prefix);
+  self->prefix = g_strdup(prefix);
+}
+
+void
+journal_reader_options_set_max_field_size(JournalReaderOptions *self, gint max_field_size)
+{
+  self->max_field_size = max_field_size;
+}
+
+void
+journal_reader_options_set_namespace(JournalReaderOptions *self, gchar *namespace)
+{
+  if (self->namespace)
+    g_free(self->namespace);
+  self->namespace = g_strdup(namespace);
+}
+
+void
+journal_reader_options_set_log_fetch_limit(JournalReaderOptions *self, gint log_fetch_limit)
+{
+  self->fetch_limit = log_fetch_limit;
+}
+
 void
 journal_reader_options_defaults(JournalReaderOptions *options)
 {
   log_source_options_defaults(&options->super);
   options->super.stats_level = STATS_LEVEL0;
-  options->super.stats_source = SCS_JOURNALD;
+  options->super.stats_source = stats_register_type("journald");
   options->fetch_limit = DEFAULT_FETCH_LIMIT;
   options->default_pri = DEFAULT_PRIO;
   options->max_field_size = DEFAULT_FIELD_SIZE;
@@ -758,5 +884,10 @@ journal_reader_options_destroy(JournalReaderOptions *options)
       time_zone_info_free(options->recv_time_zone_info);
       options->recv_time_zone_info = NULL;
     }
+  if (options->namespace)
+    {
+      g_free(options->namespace);
+      options->namespace = NULL;
+    }
   options->initialized = FALSE;
 }
